/* Toolbar + graph */

.now {
  /*background: #ccc;*/
  overflow: hidden !important;
}

/* Toolbar */

.now .toolbar {
  margin: 5px;
}

.now .toolbar a.toollink,
.now .toolbar a.toollink:link,
.now .toolbar a.toollink:visited {
  color: black;
}

.now .toolbar a.toollink:active  {
  color: blue;
}

.now .toolbar {
  float: left;
  overflow: none;
  width: 252px;
  height: auto;
  z-index: 3;
  position: absolute;
}

.now .toolbar input[type=checkbox] {
    display: none
}

.now .toolbar label, .now .toolbar a {
    margin-right: 8px;
}

.now .toolbar input[type=checkbox] + label {
    display:inline-block;
    color: black;
    cursor: pointer;
}

.now .toolbar input[type=checkbox]:checked + label {
    color: blue;
}

/* Graph */

.now svg {
  background-color: #fff;
  cursor: default;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
}

.now svg:not(.active):not(.ctrl) {
  cursor: crosshair;
}

/* Tooltip */

div.now-tooltip {
    position: absolute;
    left: 1px;
    right: 1px;
    max-height: 200px;
    max-width: 400px;
    overflow: auto;
    padding: 5px;
    padding-bottom: 10px;
    font: 12px sans-serif;
    background: #DDD;
    border: 0px;
    border-radius: 8px;
    z-index: 2000;
}

.now-tooltip.hidden {
    display: none;
}

.ipython-graph {
}

.ipython-graph:hover {
}

.hbefore {
  fill: red !important;
  font-size: 20px !important;
}

.hafter {
  fill: green !important;
  font-size: 20px !important;
}

.bottom .ipython-graph {
  display: inline-block;
}

.node-hover {
  stroke: blue !important;
}

/* Link Button */

a.link-button {
  text-decoration: none;
  display:inline-block;
  margin: 3px -1px;
  padding: 4px 0;
 /*padding: 4px 12px;*/
  margin-bottom: 0;
  font-size: 14px;
  width: 100%;
  line-height: 15px;
  color: #333;
  text-align: center;
  text-shadow: 0 1px 1px rgba(255,255,255,0.75);
  vertical-align: middle;
  cursor: pointer;
  background-color: #f5f5f5;
  background-image: -moz-linear-gradient(top,#fff,#e6e6e6);
  background-image: -webkit-gradient(linear,0 0,0 100%,from(#fff),to(#e6e6e6));
  background-image: -webkit-linear-gradient(top,#fff,#e6e6e6);
  background-image: -o-linear-gradient(top,#fff,#e6e6e6);
  background-image: linear-gradient(to bottom,#fff,#e6e6e6);
  background-repeat: repeat-x;
  border: 1px solid #ccc;
  border-color: #e6e6e6 #e6e6e6 #bfbfbf;
  border-color: rgba(0,0,0,0.1) rgba(0,0,0,0.1) rgba(0,0,0,0.25);
  border-bottom-color: #b3b3b3;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffffff',endColorstr='#ffe6e6e6',GradientType=0);
  filter: progid:DXImageTransform.Microsoft.gradient(enabled=false);
  -webkit-box-shadow: inset 0 1px 0 rgba(255,255,255,0.2),0 1px 2px rgba(0,0,0,0.05);
  -moz-box-shadow: inset 0 1px 0 rgba(255,255,255,0.2),0 1px 2px rgba(0,0,0,0.05);
  box-shadow: inset 0 1px 0 rgba(255,255,255,0.2),0 1px 2px rgba(0,0,0,0.05);
}

a.link-button:active {
  background-image: none;
  outline: 0;
  -webkit-box-shadow: inset 0 2px 4px rgba(0,0,0,0.15),0 1px 2px rgba(0,0,0,0.05);
  -moz-box-shadow: inset 0 2px 4px rgba(0,0,0,0.15),0 1px 2px rgba(0,0,0,0.05);
  box-shadow: inset 0 2px 4px rgba(0,0,0,0.15),0 1px 2px rgba(0,0,0,0.05);
  background-color:#e0e0e0;
}

/* Trial graph type (exact match x combined) */

.select-style {
  border: 1px solid #ccc;
  width: 230px;
  height: 30px;
  border-radius: 3px;
  overflow: hidden;
  background: #fafafa url("img/icon-select.png") no-repeat 90% 50%;
}

.select-style select {
  padding: 5px 8px;
  width: 130%;
  border: none;
  box-shadow: none;
  background: transparent;
  background-image: none;
  -webkit-appearance: none;
}

.select-style select:focus {
  outline: none;
}

#graphselector {
  width: calc(100% - 20px);
  padding: 10px;
  border-bottom: 1px solid black;
}

#graphselector #use_cache {
  margin-top: 10px;
  margin-left: 0px;
}

#graphselector a.link-button {
  margin-top: 10px;
  width: 338px;
}

#graphselector label {
  font-size: 14px;
}

.graph-attr {
  display: block;
}

#show-graph-toolbar:hover {
  cursor: pointer;
}

#graphbox {
  overflow: hidden;
}

#graphselector .graph-attr:first-child label {
  margin-right: 17px;
}

#loadingDiv {
  position:absolute;
  top:0;
  right:0;
  font-size: 70px;
  margin: 5px;
}