div.now-history-tooltip {
    max-width: 250px;
}

.now-history-graph svg {

}

.now-history-graph path.link {
  fill: none;
  stroke: #000;
  stroke-width: 4px;
  cursor: default;
}

.now-history-graph svg:not(.active):not(.ctrl) path.link {
  cursor: crosshair;
}

.now-history-graph path.link.selected {
  stroke-dasharray: 10,2;
}

.now-history-graph path.link.dragline {
  pointer-events: none;
}

.now-history-graph path.link.hidden {
  stroke-width: 0;
}

.now-history-graph circle.node {
  stroke-width: 1.5px;
  cursor: pointer;
  stroke: #333;
  stroke-width: 2px;
}

.now-history-graph circle.unfinished {
  fill: rgb(238, 200, 241);
}

.now-history-graph circle.finished {
  fill: #F6FBFF;
}

.now-history-graph circle.backup {
  fill: rgb(241, 238, 200);
}

.now-history-graph circle.node:hover {
  stroke-width: 1.5px;
  cursor: pointer;
  fill: rgb(200, 238, 241);
  stroke: #333;
  stroke-width: 2px;
}

.now-history-graph circle.selected {
  fill: rgb(200, 238, 241) !important;
}

.now-history-graph circle.node.reflexive {
  stroke: #000 !important;
  stroke-width: 2.5px;
}

.now-history-graph text {
  font: 12px sans-serif;
  pointer-events: none;
}

.now-history-graph text.id {
  text-anchor: middle;
  font-weight: bold;
}
