.dbefore {
  color: red !important;
}

.dafter {
  color: green !important;
}

.dbefore:before {
  color: red !important;
  content: ' - ';
  font-family: "Courier New", Courier, monospace;
}


.dafter:before {
  color: green !important;
  content: ' + ';
  font-family: "Courier New", Courier, monospace;
}

.libefore {
  border-left: 3px solid red !important;
  color: red !important;
}

.liafter {
  border-left: 3px solid green !important;
  color: green !important;
}

#graphA, #graphB, #graph {
  overflow: hidden !important;
}

.now .toolbar {
  margin: 5px;
}



#graphselector .graph-attr:first-child {
  margin-bottom: 5px;
}

.time-attr {
  margin-top: 5px;
}

.time-attr label {
  margin-right: 25px;
}

.time-attr input {
  padding-left: 4px;
}